@@ -10,6 +10,7 @@
 #include "Configuration.hpp"
 #include "Log.hpp"
 #include "Mount.hpp"
+#include "Snapshot.hpp"
 #include "Util.hpp"
 #include <cstring>
 #include <filesystem>
@@ -17,6 +18,7 @@
 #include <selinux/selinux.h>
 #include <selinux/context.h>
 #include <sstream>
+#include <sys/stat.h>
 #include <unistd.h>
 
 using std::exception;
@@ -37,7 +39,7 @@ namespace TransactionalUpdate {
 Overlay::Overlay(string snapshot):
         workdir(fs::path{config.get("OVERLAY_DIR")} / snapshot / "work-etc")
 {
-    fs::create_directories(fs::path{workdir});
+    fs::create_directories(workdir);
 
     // Read lowerdirs
     // Always use the root file system for reading fstab, as the first snapshot overlay doesn't
@@ -185,7 +187,16 @@ void Overlay::create(string base, string snapshot, fs::path snapRoot) {
 
     // Remove overlay directory if it already exists (e.g. after the snapshot was deleted)
     fs::remove_all(upperdir);
-    fs::create_directories(upperdir);
+    fs::create_directory(upperdir, "/etc");
+    // Explicitly copy the current /etc permissions, even if the umask was more restrictive
+    struct stat status;
+    tulog.info(parent.upperdir.string());
+    if (stat(parent.upperdir.c_str(), &status) != 0) {
+        throw std::runtime_error{"could not get permissions of /etc: " + std::string(strerror(errno))};
+    }
+    if (chmod(upperdir.c_str(), status.st_mode) != 0) {
+        throw std::runtime_error{"could not set permissions of " + upperdir.string() + ": " + std::string(strerror(errno))};
+    }
 
     char* context = NULL;
     if (getfilecon("/etc", &context) > 0) {
