@@ -17,7 +17,7 @@ namespace TransactionalUpdate {
 std::unique_ptr<Snapshot> Snapper::create(std::string base, std::string description) {
     if (! std::filesystem::exists("/.snapshots/" + base + "/snapshot"))
         throw std::invalid_argument{"Base snapshot '" + base + "' does not exist."};
-    snapshotId = callSnapper("create --from " + base + " --read-write --print-number --description '" + description + "' --userdata 'transactional-update-in-progress=yes'");
+    snapshotId = callSnapper("create --from " + base + " --read-write --cleanup-algorithm number --print-number --description '" + description + "' --userdata 'transactional-update-in-progress=yes'");
     Util::rtrim(snapshotId);
     return std::make_unique<Snapper>(snapshotId);
 }
@@ -125,6 +125,10 @@ void Snapper::deleteSnap(std::string id) {
     callSnapper("delete " + id);
 }
 
+void Snapper::rollbackTo(std::string id) {
+    callSnapper("rollback " + id);
+}
+
 bool Snapper::isInProgress() {
     std::string desc = callSnapper("--csvout list --columns number,userdata");
     std::smatch match;
@@ -132,9 +136,12 @@ bool Snapper::isInProgress() {
 }
 
 bool Snapper::isReadOnly() {
-    std::string ro = Util::exec("btrfs property get " + std::string(getRoot()) + " ro");
-    Util::rtrim(ro);
-    if (ro == "ro=true")
+    std::string ro = callSnapper("--csvout list --columns number,read-only");
+    std::smatch match;
+    bool found = std::regex_search(ro, match, std::regex(snapshotId + ",(.*)"));
+    if (!found)
+        throw std::runtime_error{"Couldn't determine read-only state"};
+    if (match[1].str() == "yes")
         return true;
     else
         return false;
@@ -142,8 +149,8 @@ bool Snapper::isReadOnly() {
 
 void Snapper::setDefault() {
     try {
-        callSnapper("modify --default " + snapshotId);
-    } catch (const ExecutionException &e) {
+        callSnapper("modify --default " + snapshotId + " 2>&1");
+    } catch (const VersionException &e) {
         Util::exec("btrfs subvolume set-default " + std::string(getRoot()));
     }
 }
@@ -151,10 +158,10 @@ void Snapper::setDefault() {
 void Snapper::setReadOnly(bool readonly) {
     try {
         if (readonly == true)
-            callSnapper("modify --read-only " + snapshotId);
+            callSnapper("modify --read-only " + snapshotId + " 2>&1");
         else
-            callSnapper("modify --read-write " + snapshotId);
-    } catch (const ExecutionException &e) {
+            callSnapper("modify --read-write " + snapshotId + " 2>&1");
+    } catch (const VersionException &e) {
         Util::exec("btrfs property set " + std::string(getRoot()) + " ro " + (readonly ? "true" : "false"));
     }
 }
@@ -162,17 +169,23 @@ void Snapper::setReadOnly(bool readonly) {
 /* Helper methods */
 
 std::string Snapper::callSnapper(std::string opts) {
-    if (snapperNoDbus == false) {
-        try {
-            return Util::exec("snapper " + opts);
-        } catch (const ExecutionException &e) {
-            snapperNoDbus = true;
+    std::string output;
+    try {
+        if (std::filesystem::exists("/run/dbus/system_bus_socket")) {
+            output = Util::exec("snapper " + opts);
+        } else {
+            output = Util::exec("snapper --no-dbus " + opts);
+        }
+    } catch (const ExecutionException &e) {
+        if (e.output.rfind("Unknown option", 0) == 0) {
+            std::string message;
+            std::getline(std::istringstream(e.output), message);
+            throw VersionException{"snapper: " + message};
+        } else {
+            throw;
         }
     }
-    if (snapperNoDbus == true) {
-        return Util::exec("snapper --no-dbus " + opts);
-    }
-    return "false ";
+    return output;
 }
 
 } // namespace TransactionalUpdate
