@@ -17,7 +17,6 @@
 #include "Supplement.hpp"
 #include "Util.hpp"
 #include <cerrno>
-#include <chrono>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
@@ -48,7 +47,7 @@ public:
     int inotifyRead();
     std::unique_ptr<SnapshotManager> snapshotMgr;
     std::unique_ptr<Snapshot> snapshot;
-    std::string bindDir;
+    fs::path bindDir;
     std::vector<std::unique_ptr<Mount>> dirsToMount;
     Supplements supplements;
     pid_t pidCmd;
@@ -72,7 +71,7 @@ Transaction::~Transaction() {
     pImpl->dirsToMount.clear();
     if (!pImpl->bindDir.empty()) {
         try {
-            fs::remove(fs::path{pImpl->bindDir});
+            fs::remove(pImpl->bindDir);
         }  catch (const std::exception &e) {
             tulog.error("ERROR: ", e.what());
         }
@@ -140,7 +139,7 @@ void Transaction::impl::snapMount() {
                 throw std::runtime_error{"Forking for SELinux relabelling failed: " + std::string(strerror(errno))};
             } else if (childPid == 0) {
                 if (chroot(bindDir.c_str()) < 0) {
-                    tulog.error("Chrooting to " + bindDir + " for SELinux relabelling failed: " + std::string(strerror(errno)));
+                    tulog.error("Chrooting to " + bindDir.native() + " for SELinux relabelling failed: " + std::string(strerror(errno)));
                     _exit(errno);
                 }
                 unsigned int restoreconOptions = SELINUX_RESTORECON_RECURSE | SELINUX_RESTORECON_IGNORE_DIGEST;
@@ -370,15 +369,18 @@ int Transaction::impl::runCommand(char* argv[], bool inChroot, std::string* outp
         if (output != nullptr) {
             ret = dup2(pipefd[1], STDOUT_FILENO);
             if (ret < 0) {
-                throw std::runtime_error{"Redirecting stdout failed: " + std::string(strerror(errno))};
+                tulog.error("Redirecting stdout failed: " + std::string(strerror(errno)));
+                _exit(errno);
             }
             ret = dup2(pipefd[1], STDERR_FILENO);
             if (ret < 0) {
-                throw std::runtime_error{"Redirecting stderr failed: " + std::string(strerror(errno))};
+                tulog.error("Redirecting stderr failed: " + std::string(strerror(errno)));
+                _exit(errno);
             }
             ret = close(pipefd[0]);
             if (ret < 0) {
-                throw std::runtime_error{"Closing pipefd failed: " + std::string(strerror(errno))};
+                tulog.error("Closing pipefd failed: " + std::string(strerror(errno)));
+                _exit(errno);
             }
         }
 
@@ -387,28 +389,34 @@ int Transaction::impl::runCommand(char* argv[], bool inChroot, std::string* outp
                 tulog.info("Warning: Couldn't set working directory: ", std::string(strerror(errno)));
             }
             if (chroot(bindDir.c_str()) < 0) {
-                throw std::runtime_error{"Chrooting to " + bindDir + " failed: " + std::string(strerror(errno))};
+                tulog.error("Chrooting to " + bindDir.native() + " failed: " + std::string(strerror(errno)));
+                _exit(errno);
             }
             // Prevent mounts from within the chroot environment influence the tukit organized mounts
             if (unshare(CLONE_NEWNS) < 0) {
-                throw std::runtime_error{"Creating new mount namespace failed: " + std::string(strerror(errno))};
+                tulog.error("Creating new mount namespace failed: " + std::string(strerror(errno)));
+                _exit(errno);
             }
             if (mount("none", "/", NULL, MS_REC|MS_PRIVATE, NULL) < 0) {
-                 throw std::runtime_error{"Setting private mount for command execution failed: " + std::string(strerror(errno))};
+                tulog.error("Setting private mount for command execution failed: " + std::string(strerror(errno)));
+                _exit(errno);
             }
         }
 
         // Set indicator for RPM pre/post sections to detect whether we run in a
         // transactional update
         if (setenv("TRANSACTIONAL_UPDATE", "true", 1) < 0) {
-            throw std::runtime_error{"Setting environment variable TRANSACTIONAL_UPDATE failed: " + std::string(strerror(errno))};
+            tulog.error("Setting environment variable TRANSACTIONAL_UPDATE failed: " + std::string(strerror(errno)));
+            _exit(errno);
         }
         if (setenv("TRANSACTIONAL_UPDATE_ROOT", snapshot->getRoot().c_str(), 1)) {
-            throw std::runtime_error{"Setting environment variable TRANSACTIONAL_UPDATE_ROOT failed: " + std::string(strerror(errno))};
+            tulog.error("Setting environment variable TRANSACTIONAL_UPDATE_ROOT failed: " + std::string(strerror(errno)));
+            _exit(errno);
         }
 
         if (execvp(argv[0], (char* const*)argv) < 0) {
-            throw std::runtime_error{"Calling " + std::string(argv[0]) + " failed: " + std::string(strerror(errno))};
+            tulog.error("Calling " + std::string(argv[0]) + " failed: " + std::string(strerror(errno)));
+            _exit(errno);
         }
         ret = -1;
     } else {
@@ -453,8 +461,8 @@ int Transaction::callExt(char* argv[], std::string* output) {
         std::string from = "{}";
         // replacing all {} by bindDir
         for(size_t pos = 0;
-           (pos = s.find(from, pos)) != std::string::npos;
-           pos += getRoot().string().length())
+            (pos = s.find(from, pos)) != std::string::npos;
+            pos += getRoot().string().length())
             s.replace(pos, from.size(), this->pImpl->bindDir);
         argv[i] = strdup(s.c_str());
     }
@@ -500,7 +508,7 @@ void Transaction::finalize() {
                 targetRoot = previousOvl.upperdir.parent_path() / "sync";
                 previousEtc->mount(targetRoot);
             }
-            Util::exec("rsync --archive --inplace --xattrs --acls --exclude 'fstab' --delete --quiet '" + this->pImpl->bindDir + "/etc/' " + targetRoot.native() + "/etc");
+            Util::exec("rsync --archive --inplace --xattrs --acls --exclude 'fstab' --delete --quiet '" + this->pImpl->bindDir.native() + "/etc/' " + targetRoot.native() + "/etc");
         }
 
         return;
